@@ -3,13 +3,22 @@
 capabilities.py
 
 Created by Thomas Mangin on 2012-07-17.
-Copyright (c) 2009-2015 Exa Networks. All rights reserved.
+Copyright (c) 2009-2017 Exa Networks. All rights reserved.
+License: 3-clause BSD. (See the COPYRIGHT file)
 """
 
+from exabgp.vendoring import six
+
 from exabgp.protocol.family import AFI
 from exabgp.protocol.family import SAFI
 
+from exabgp.util import character
+from exabgp.util import ordinal
+from exabgp.util import concat_bytes
+from exabgp.util import concat_bytes_i
+
 from exabgp.bgp.message.open.capability.capability import Capability
+from exabgp.bgp.message.open.capability.nexthop import NextHop
 from exabgp.bgp.message.open.capability.addpath import AddPath
 from exabgp.bgp.message.open.capability.asn4 import ASN4
 from exabgp.bgp.message.open.capability.graceful import Graceful
@@ -18,7 +27,8 @@ from exabgp.bgp.message.open.capability.ms import MultiSession
 from exabgp.bgp.message.open.capability.operational import Operational
 from exabgp.bgp.message.open.capability.refresh import RouteRefresh
 from exabgp.bgp.message.open.capability.refresh import EnhancedRouteRefresh
-# from exabgp.bgp.message.open.capability.unknown import UnknownCapability
+from exabgp.bgp.message.open.capability.extended import ExtendedMessage
+from exabgp.bgp.message.open.capability.hostname import HostName
 
 from exabgp.bgp.message.notification import Notify
 
@@ -38,7 +48,7 @@ class Parameter (int):
 		return 'UNKNOWN'
 
 # =================================================================== Capabilities
-# http://www.iana.org/assignments/capability-codes/
+# https://www.iana.org/assignments/capability-codes/
 
 # +------------------------------+
 # | Capability Code (1 octet)    |
@@ -51,11 +61,18 @@ class Parameter (int):
 
 class Capabilities (dict):
 	_ADD_PATH = [
-		(AFI(AFI.ipv4),SAFI(SAFI.unicast)),
-		(AFI(AFI.ipv6),SAFI(SAFI.unicast)),
-		(AFI(AFI.ipv4),SAFI(SAFI.nlri_mpls)),
-		(AFI(AFI.ipv4),SAFI(SAFI.mpls_vpn)),
-		(AFI(AFI.ipv6),SAFI(SAFI.mpls_vpn)),
+		(AFI.ipv4,SAFI.unicast),
+		(AFI.ipv6,SAFI.unicast),
+		(AFI.ipv4,SAFI.nlri_mpls),
+		(AFI.ipv4,SAFI.mpls_vpn),
+		(AFI.ipv6,SAFI.mpls_vpn),
+	]
+
+	_NEXTHOP = [
+		(AFI.ipv4, SAFI.unicast, AFI.ipv6),
+		(AFI.ipv4, SAFI.multicast, AFI.ipv6),
+		(AFI.ipv4, SAFI.nlri_mpls, AFI.ipv6),
+		(AFI.ipv4, SAFI.mpls_vpn ,AFI.ipv6),
 	]
 
 	def announced (self, capability):
@@ -79,11 +96,23 @@ class Capabilities (dict):
 
 		self[Capability.CODE.FOUR_BYTES_ASN] = ASN4(neighbor.local_as)
 
+	def _nexthop (self, neighbor):
+		if not neighbor.nexthop:
+			return
+
+		nexthops = neighbor.nexthops()
+		nh_pairs = []
+		for allowed in self._NEXTHOP:
+			if allowed not in nexthops:
+				continue
+			nh_pairs.append(allowed)
+		self[Capability.CODE.NEXTHOP] = NextHop(nh_pairs)
+
 	def _addpath (self, neighbor):
 		if not neighbor.add_path:
 			return
 
-		families = neighbor.families()
+		families = neighbor.addpaths()
 		ap_families = []
 		for allowed in self._ADD_PATH:
 			if allowed in families:
@@ -106,6 +135,15 @@ class Capabilities (dict):
 		self[Capability.CODE.ROUTE_REFRESH] = RouteRefresh()
 		self[Capability.CODE.ENHANCED_ROUTE_REFRESH] = EnhancedRouteRefresh()
 
+	def _extended_message (self, neighbor):
+		if not neighbor.extended_message:
+			return
+
+		self[Capability.CODE.EXTENDED_MESSAGE] = ExtendedMessage()
+
+	def _hostname (self, neighbor):
+		self[Capability.CODE.HOSTNAME] = HostName(neighbor.host_name,neighbor.domain_name)
+
 	def _operational (self, neighbor):
 		if not neighbor.operational:
 			return
@@ -120,45 +158,48 @@ class Capabilities (dict):
 	def new (self, neighbor, restarted):
 		self._protocol(neighbor)
 		self._asn4(neighbor)
+		self._nexthop(neighbor)
 		self._addpath(neighbor)
 		self._graceful(neighbor,restarted)
 		self._refresh(neighbor)
 		self._operational(neighbor)
+		self._extended_message(neighbor)
+		# self._hostname(neighbor)  # Cumulus draft - disabling until -01 is out
 		self._session(neighbor)  # MUST be the last key added, really !?! dict is not ordered !
 		return self
 
 	def pack (self):
 		rs = []
-		for k,capabilities in self.iteritems():
+		for k,capabilities in six.iteritems(self):
 			for capability in capabilities.extract():
-				rs.append("%s%s%s" % (chr(k),chr(len(capability)),capability))
-		parameters = "".join(["%s%s%s" % (chr(2),chr(len(r)),r) for r in rs])
-		return "%s%s" % (chr(len(parameters)),parameters)
+				rs.append(concat_bytes(character(k),character(len(capability)),capability))
+		parameters = concat_bytes_i(concat_bytes(character(2),character(len(r)),r) for r in rs)
+		return concat_bytes(character(len(parameters)),parameters)
 
 	@staticmethod
 	def unpack (data):
 		def _key_values (name, data):
 			if len(data) < 2:
 				raise Notify(2,0,"Bad length for OPEN %s (<2) %s" % (name,Capability.hex(data)))
-			l = ord(data[1])
-			boundary = l+2
+			ld = ordinal(data[1])
+			boundary = ld+2
 			if len(data) < boundary:
 				raise Notify(2,0,"Bad length for OPEN %s (buffer underrun) %s" % (name,Capability.hex(data)))
-			key = ord(data[0])
+			key = ordinal(data[0])
 			value = data[2:boundary]
 			rest = data[boundary:]
 			return key,value,rest
 
 		capabilities = Capabilities()
 
-		option_len = ord(data[0])
+		option_len = ordinal(data[0])
 		if not option_len:
 			return capabilities
 
 		data = data[1:option_len+1]
 		while data:
 			key,value,data = _key_values('parameter',data)
-			# Paramaters must only be sent once.
+			# Parameters must only be sent once.
 			if key == Parameter.AUTHENTIFICATION_INFORMATION:
 				raise Notify(2,5)
 
