@@ -1,6 +1,7 @@
 import os
 
-release = "4.1.2-d006a34a"
+commit = "99de31e8"
+release = "4.2.6"
 json = "4.0.1"
 text = "4.0.1"
 version = os.environ.get('EXABGP_VERSION',release)
